/* 
   header file for ccam system (xext, ccam, stat)
   ccam.h ver 4.00
   (c) Junchi Akita (akita@fun.ac.jp)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/mman.h>
#define _POSIX_SOURCE 1

#define USE_OBJ_LUT
#define OBJ_LUT_SIZE 128

extern unsigned char f[OBJ_LUT_SIZE][OBJ_LUT_SIZE][OBJ_LUT_SIZE];
extern unsigned int fl[OBJ_LUT_SIZE][OBJ_LUT_SIZE][OBJ_LUT_SIZE];

extern int XS, XE, YS, YE; /* lower-left & upper-right corner of field */
extern int XA, YA;         /* width & height of field                  */
typedef struct{
  unsigned int r, g, b;
  double h, s, v;
} colors;
extern colors calc_color(colors col); /* color system conversion */

#define H_ORIGIN (-45) /* Hue origin [deg.] */

/* variables for location, robots, etc. */
extern int frame;                    /* frame number processed              */
extern int Nm;                       /* number of markers specified         */
#define CB   (1)                     /* object number of ball               */
#define CYM  (2)                     /* object number of Y-Ping             */
#define CBM  (3)                     /* object number of B-Ping             */
#define CYM1 (4)                     /* object number of Y-Marker1          */
#define CYM2 (5)                     /* object number of Y-Marker2          */
#define CBM1 (6)                     /* object number of B-Marker1          */
#define CBM2 (7)                     /* object number of B-Marker2          */
#define DK (-1)                      /* coordinate for non-object point     */

#define TFRAME (50) /* frame cycle [ms] */

extern unsigned char *dbuf;

extern colors get_rgb(unsigned char *bf, colors col);
extern unsigned char *set_rgb(unsigned char *bf, colors col);
extern unsigned char *set_buf(unsigned char *bf, int x, int y);

extern colors calc_color(colors col);
colors calc_hsv_to_rgb(colors col);
extern unsigned char ***get_lut(unsigned char ***f, FILE *fp);

extern int init_socket(int fd, char *hostname, int rport);
extern void send_string(int fd, char *buf0);
extern void close_socket(int fd);
extern hostent *hp;

extern int serial_init(int fd);

#ifdef LARGE_V4L
#define X (640)
#define Y (480)
#else
#define X (320)
#define Y (240)
#endif

#define DIV (256 / OBJ_LUT_SIZE)

#define FIELD_SIZE_FILENAME "./size.txt"   /* field size description file */
#define ROBOT_SHAPE_FILENAME "./shape.txt" /* robot description file */

#define NRMAX  15   // allowable maximum number of robots per team
#define NMKMAX 30   // allowable maximum number of markers in image
