/*
  Color conversion functions
  (c) Junichi Akita (akita@fun.ac.jp)
 */

#include "ccam.h"

colors calc_color(colors col)
{
  double r, g, b, cmax, cmin, cd;
  double s, rn, gn, bn;

  r = (double)col.r / 255.0;
  g = (double)col.g / 255.0;
  b = (double)col.b / 255.0;

  if (r >= g) cmax = r; else cmax = g; if (b > cmax) cmax = b;
  if (r <= g) cmin = r; else cmin = g; if (b < cmin) cmin = b;
  col.v = cmax;
  cd = cmax - cmin;
  if (cd == 0.0){col.s = 0.0; col.h = 0.0;} else{col.s = cd / cmax;}
  if (cd > 0.001){
    if (r == cmax) col.h = 0.0 + (g - b) / cd;
    if (g == cmax) col.h = 2.0 + (b - r) / cd;
    if (b == cmax) col.h = 4.0 + (r - g) / cd;
    col.h *= 60.0;if (col.h < 0.0) col.h += 360.0;
  }
  else col.h = 0.0;
  while(col.h < H_ORIGIN) col.h += 360.0;
  while(col.h > H_ORIGIN+360) col.h -= 360.0;
  return(col);
}

colors calc_hsv_to_rgb(colors col)
{
  double h, s, v;
  double p1, p2, p3;
  double f, i;
  double r, g, b;

  h = col.h; s = col.s; v = col.v;
  /* h : 0 - 360, s&v : 0.0 - 1.0 */

  h = fmod(h, 360); if (h < 0) h += 360;
  h /= 60;
  f = modf(h, &i);

  p1 = v * (1-s);
  p2 = v * (1-s*f);
  p3 = v * (1-s*(1-f));

  switch((int)i)
    {
    case 0: r = v;  g = p3; b = p1;
    case 1: r = p2; g = v;  b = p1;
    case 2: r = p1; g = v;  b = p3;
    case 3: r = p1; g = p2; b = v;
    case 4: r = p3; g = p1; b = v;
    case 5: r = v;  g = p1; b = p2;
    }
  col.r = (int)(255*r);
  col.g = (int)(255*g);
  col.b = (int)(255*b);
  return(col);
}

unsigned char ***get_lut(unsigned char ***f, FILE *fp)
{
  int r, g, b;
  colors col;

  f = (unsigned char ***)malloc(sizeof(unsigned char **) * OBJ_LUT_SIZE);
  for (r = 0; r < OBJ_LUT_SIZE; r++)
    f[r] = (unsigned char **)malloc(sizeof(unsigned char *) * OBJ_LUT_SIZE);
  for (r = 0; r < OBJ_LUT_SIZE; r++)
    for (g = 0; g < OBJ_LUT_SIZE; g++)
      f[r][g] = (unsigned char *)malloc(sizeof(unsigned char) * OBJ_LUT_SIZE);
  for (r = 0; r < OBJ_LUT_SIZE; r++)
    for (g = 0; g < OBJ_LUT_SIZE; g++)
      for (b = 0; b < OBJ_LUT_SIZE; b++)
	f[r][g][b] = (unsigned char)fgetc(fp);
  return(f);

}

