/*
  kpad: Keyboard Pad Controller for ssend
   (c) Junichi Akita (akita@fun.ac.jp)
   ver 4.00

   Robot Motion: cursor-key or ten-key or vi-mode
    Forward   : Cursor-UP    or '8' or 'k'
    Backward  : Cursor-DOWN  or '2' or 'j'
    LeftTurn  : Cursor-LEFT  or '4' or 'h'
    RightTurn : Cursor-RIGHT or '6' or 'l'
    Stop      : Space        or '5' or 'm'
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <termio.h>
#include <math.h>
#include <stdlib.h>

#define PORT (10031) /* Port number for sending UDP packet to target */

int getch(void)
{
  struct termio orig, work;
  int i;
    
  ioctl(0,TCGETA,&orig);
  work = orig;
  work.c_lflag &=~ICANON;
  work.c_cc[VMIN] = 1;
  work.c_cc[VTIME] = 0;
  work.c_lflag &=~ECHO;
  ioctl(0,TCSETAW,&work);
  i = getchar();
  ioctl(0,TCSETA,&orig);
  return i;
}

#define NO 0
#define FW 1
#define BK 2
#define LT 3
#define RT 4
#define ST 5

main(int argc, char *argv[])
{
  struct sockaddr_in    saddr;
  struct hostent *hp;
  int fd, len, buflen, ret, port;
  char buf[1024];
  char com[5];
  int rn;
  double speed;
  char motion;

  if (argc == 4) port = PORT;
  else if (argc == 5) port = atoi(argv[4]);
  else{
    printf("Usage: %s ROBOT_ID SPEED SERVER_NAME [PORT]\n", argv[0]);
    exit(1);
  }
  rn = atoi(argv[1]);
  speed = atof(argv[2]);

  if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "socket\n"); exit(1);
  }

  bzero((char *)&saddr, sizeof(saddr));
  
  if ((hp = gethostbyname(argv[3])) == NULL) {
    fprintf(stderr, "No such host\n"); exit(1);
  }
  bcopy(hp->h_addr, &saddr.sin_addr, hp->h_length);
  saddr.sin_family = AF_INET; saddr.sin_port = htons(port);

  motion = NO;
  while(1){
    switch(getch()){
    case '8': motion = FW; break;
    case '2': motion = BK; break;
    case '4': motion = LT; break;
    case '6': motion = RT; break;
    case '5': motion = ST; break;
    case 'k': motion = FW; break;
    case 'j': motion = BK; break;
    case 'l': motion = LT; break;
    case 'h': motion = RT; break;
    case 'm': motion = ST; break;
    case ' ': motion = ST; break;
    case 0x1b:{
      if (getch() == 0x5b){
	switch(getch()){
	case 0x41: motion = FW; break;
	case 0x42: motion = BK; break;
	case 0x43: motion = RT; break;
	case 0x44: motion = LT; break;
	default: motion = NO; break;
	}
      }
      break;
    }
    default: motion = NO; break;
    }
    switch(motion){
    case FW: sprintf(buf, "%d %.2f %.2f\n", rn,  speed,  speed); break;
    case BK: sprintf(buf, "%d %.2f %.2f\n", rn, -speed, -speed); break;
    case LT: sprintf(buf, "%d %.2f %.2f\n", rn, -speed,  speed); break;
    case RT: sprintf(buf, "%d %.2f %.2f\n", rn,  speed, -speed); break;
    case ST: sprintf(buf, "%d %.2f %.2f\n", rn,  0.0, 0.0); break;
    default: break;
    }
    if (motion != NO){
      buflen = strlen(buf);
      printf("%s", buf);
      if (sendto(fd, buf, buflen, 0,
		 (struct sockaddr *)&saddr, sizeof(saddr)) != buflen){
	fprintf(stderr, "sendto\n");
	exit(1);
      }
    }
  }
  close(fd);
}
