/*
  Sample Program for RoboCup
    using 'ccam' and 'ssend'
    (c) Junichi Akita (akita@fun.ac.jp), 2003.4.16
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <strings.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/time.h>

#define REQUESTMESSAGE_LENGTH 64
#define HOSTNAME_LENGTH 64
#define COMMAND_LENGTH 64
#define TEMP_RECEIVE_LENGTH 1024
#define BUFFER_RECEIVE_LENGTH 1024
#define BUFFER_SEND_LENGTH 64
#define INFORMATION_NUM 37

#define CMD_CYCLE 100  /* command send cycle time [ms] */
/*
#define DEBUG
*/

#define QPORT 10090

//#define TEAM "Y"
#define TEAM "B"

main(int ac, char **av)
{
  int csock, ssock, qsock;
  struct sockaddr_in c_addr, s_addr, q_addr;
  struct hostent *hp_local;
  struct hostent *hp_c, *hp_s;
  struct in_addr ia;
  char cbuf[1024], sbuf[1024], qbuf[1024];
  int cbuflen, sbuflen, qbuflen, buflen;
  int cport, sport;
  int id, pos;
  char chost[1024], shost[1024], lhost[1024];
  struct timeval tv;
  unsigned long t0, t1;
  double L, R, L0, R0;
  int frx[5], fry[5], frdir[5];
  int erx[5], ery[5], erdir[5];
  int bx, by;
  int fx, fy;
  int time;
  int nrf, nre;
  int i;
	
  if (ac != 6){
    fprintf(stderr, "usage: CCAM_HOST CCAM_PORT SSEND_HOST SSEND_PORT ROBOT_ID ROBOT_NUM%s \n", av[0]);
    fprintf(stderr, "   CCAM_HOST  : ccam's host name\n");
    fprintf(stderr, "   CCAM_PORT  : ccam's port acceptiing request\n");
    fprintf(stderr, "   SSEND_HOST : ssend's host name ('-' if same to CCAM_HOST\n");
    fprintf(stderr, "   SSEND_PORT : ssned's port accepting motion commands\n");
    fprintf(stderr, "   ROBOT_ID   : robot's ID described in shape.txt\n");
    exit(1);
  }
  strcpy(chost, av[1]);
  cport = atoi(av[2]);
  if (strcmp(av[1], "-") == 0) strcpy(shost, chost); else strcpy(shost, av[3]);
  sport = atoi(av[4]);

  id = atoi(av[5]);

  /* intialization of sockets */
  if ((csock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
    fprintf(stderr, "failed to socket().\n"); exit(1);
  }
  bzero((char *)&c_addr, sizeof(c_addr));
  if ((hp_c = gethostbyname(chost)) == NULL){
    fprintf(stderr, "can't find hostname of %s.\n", chost); exit(1);
  }
  bcopy(hp_c->h_addr, &c_addr.sin_addr, hp_c->h_length);
  c_addr.sin_family = AF_INET;
  c_addr.sin_port = htons(cport);

  if ((qsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
    fprintf(stderr, "failed to socket().\n"); exit(1);
    exit(1);
  }
  bzero((char *)&q_addr, sizeof(q_addr));
  q_addr.sin_family = AF_INET;
  q_addr.sin_addr.s_addr = INADDR_ANY;
  q_addr.sin_port = htons(QPORT);
  if (bind(qsock, (struct sockaddr *)&q_addr, sizeof(q_addr)) < 0){
    fprintf(stderr, "failed to bind().\n"); exit(1);
    exit(1);
  }

  if ((ssock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
    fprintf(stderr, "failed to socket().\n"); exit(1);
  }
  bzero((char *)&s_addr, sizeof(s_addr));
  if ((hp_s = gethostbyname(shost)) == NULL){
    fprintf(stderr, "can't find hostname of %s.\n", shost); exit(1);
  }
  bcopy(hp_s->h_addr, &s_addr.sin_addr, hp_s->h_length);
  s_addr.sin_family = AF_INET;
  s_addr.sin_port = htons(sport);

  gethostname(lhost, sizeof(lhost));
  hp_local = gethostbyname(lhost);
  memcpy((char *)&ia, hp_local->h_addr, hp_local->h_length);
  sprintf(qbuf, "%s %d %s %d", inet_ntoa(ia), QPORT, TEAM, id);
  qbuflen = strlen(qbuf);

  L = 0.0; R = 0.0; L0 = L; R0 = R;
  while (1){
    /* send request to ccam */
    if (sendto(csock, qbuf, qbuflen, 0,(struct sockaddr *)&c_addr,
	       sizeof(c_addr)) != qbuflen){
      fprintf(stderr, "failed to sendto().\n"); exit(1);
    }
#ifdef DEBUG
    printf("sent request to ccam: %s\n", qbuf);
#endif
    if ((buflen = recvfrom(qsock, cbuf, 1024, 0, NULL, NULL)) < 0){
      fprintf(stderr, "failed to recvfrom().\n"); exit(1);
    }
    cbuf[buflen] = '\0';
#ifdef DEBUG
    printf("received from ccam: %s", cbuf);
#endif
    /* parse received coordinate information */
    time = atoi(strtok(cbuf, " \t\n"));
    fx = atoi(strtok(NULL, " \t\n"));
    fy = atoi(strtok(NULL, " \t\n"));
    bx = atoi(strtok(NULL, " \t\n"));
    by = atoi(strtok(NULL, " \t\n"));
    nrf = atoi(strtok(NULL, " \t\n"));
    nre = atoi(strtok(NULL, " \t\n"));
    for (i = 0; i < nrf; i++){
      frx[i] = atoi(strtok(NULL, " \t\n"));
      fry[i] = atoi(strtok(NULL, " \t\n"));
      frdir[i] = atoi(strtok(NULL, " \t\n"));
    }
    for (i = 0; i < nre; i++){
      erx[i] = atoi(strtok(NULL, " \t\n"));
      ery[i] = atoi(strtok(NULL, " \t\n"));
      erdir[i] = atoi(strtok(NULL, " \t\n"));
    }

    printf("%d / %d\n", nrf, nre);
    printf("MY    : %d %d\n", frx[0], fry[0]);
    printf("other : %d %d\n", frx[1], fry[1]);

    /*
      write your strategy program here!

      given information:
        - fx, fy         : field's width /height
        - bx, by         : ball's position
        - frx[i], fry[i] : friend's robot#i's position
                           (frx[0], fry[0] is MY robot's position)
        - frdir[i]       : friend's robot#i's direction (in degree)
        - erx[j], ery[j] : enermy's robot#j's position
        - erdir[i]       : enermy's robot#i's direction (in degree)

      target information:
        - L, R           : MY robot's left and right wheel speed (-1.0 - +1.0)
                            +1.0 : max speed forward
                             0.0 : stop
                            -1.0 : max speed backward
     */
    L+=0.001; R+=0.001;

    if (( L != L0) || (R != R0)){
      sprintf(sbuf, "%d %.3f %.3f", id, L, R);
      sbuflen = strlen(sbuf);
#ifdef DEBUG
      printf("sent command to ssend: %s\n", sbuf);
#endif
      if (sendto(ssock, sbuf, sbuflen, 0, (struct sockaddr *)&s_addr,
		 sizeof(s_addr)) != sbuflen){
	fprintf(stderr, "failed to sendto().\n");
	exit(1);
      }
      L0 = L; R0 = R;
    }
    usleep(CMD_CYCLE * 1000);
  }
}
