/*
  stat ver 4.00
    (c)Junichi Akita (akita@fun.ac.jp)
*/

#include "ccam.h"
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkprivate.h>

#define TCYCLE (100) /* cycle time to update information in [ms] */

#define SHOW_ROBOT_NUMEBR_ON_PING_PONG

#define MAG (1)

/* #1:gggBBBbb, #2:RRRrrGGG */
#define C_BALL1 0x00
#define C_BALL2 0xf8

#define C_LINE1 0xff
#define C_LINE2 0xff

#define C_FIELD1 0x00
#define C_FIELD2 0x03

#define C_BACK1 0x00
#define C_BACK2 0x00

#define C_YPING1 0xe0
#define C_YPING2 0xff

#define C_BPING1 0xff
#define C_BPING2 0x03

#ifdef LARGE_V4L
#define XA (640)
#define YA (480)
#else
#define XA (320)
#define YA (240)
#endif

#define HGOAL (50)
#define WGOAL (5)

#define XW ((XA+2*WGOAL)*MAG)
#define YW (YA*MAG+1)

#define RR (10)
#define RM (6)
#define RB (6)

GtkWidget *area;
GdkImage *image;
GtkWidget *label_info;
GdkRectangle update_rect;
struct sockaddr_in sraddr;
struct sockaddr_in saddr;
struct sockaddr_in caddr;
struct hostent *hp;

int rfd, sfd;
int bx, by;
int Yx[5], Yy[5];
double Ydir[5];
int Bx[5], By[5];
double Bdir[5];
unsigned char xbuf[XW*YW*2];
int IDfirst;

#define RPORT (10029)

void draw_hline(int x1, int x2, int y, int c1, int c2)
{
  int x;
  unsigned char *bf;
  bf = xbuf + (x1 + y*XW) * 2;
  for (x = x1; x <= x1+x2; x++){
    if (x >= 0 && x <= XW && y >= 0 && y <= YW){
      *bf++ = c1; *bf++ = c2;
    }
  }
}

void draw_vline(int x, int y1, int y2, int c1, int c2)
{
  int y;
  unsigned char *bf;
  bf = xbuf + (x + y1*XW) * 2;
  for (y = y1; y <= y1+y2; y++){
    if (x >= 0 && x <= XW && y >= 0 && y <= YW){
      *bf = c1; *(bf+1) = c2; bf += XW*2;
    }
  }
}

void draw_point(int x, int y, int c1, int c2)
{
  unsigned char *bf;
  bf = xbuf + (x + y * XW) * 2;
  *bf = c1; *(bf+1) = c2;
}

#define iabs(x) (((x)>0)?(x):(-(x)))

void draw_line(int x1, int y1, int x2, int y2, int c1, int c2)
{
  int x, y;
  int swap;
  double a;

  if (iabs(x2-x1) > iabs(y2-y1)){
    if (x1 > x2){
      swap=x1;x1=x2;x2=swap;
      swap=y1;y1=y2;y2=swap;
    }
    a = (double)(y2-y1)/(double)(x2-x1);
    for (x = x1; x <= x2; x++){
      y = (int)(a * (x-x1)) + y1;
      draw_point(x, y, c1, c2);
    }
  }
  else{
    if (y1 > y2){
      swap=x1;x1=x2;x2=swap;
      swap=y1;y1=y2;y2=swap;
    }
    a = (double)(x2-x1)/(double)(y2-y1);
    for (y = y1; y <= y2; y++){
      x = (int)(a * (y-y1)) + x1;
      draw_point(x, y, c1, c2);
    }
  }
}

void draw_filled_box(int x1, int y1, int x2, int y2, int c1, int c2)
{
  unsigned char *bf;
  int x, y;

  bf = xbuf + (y1*XW + x1)*2;
  for (y = y1; y <= y1+y2; y++){
    for (x = x1; x <= x1+x2; x++){
      if (x >= 0 && x <= XW && y >= 0 && y <= YW){
	*bf++ = c1; *bf++ = c2;
      }
    }
    bf += (XW - x2 - 1)*2;
  }
}

void draw_cross(int x0, int y0, int xl, int yl, int c1, int c2)
{
  draw_hline(x0-xl/2, xl, y0, c1, c2);
  draw_vline(x0, y0-yl/2, yl, c1, c2);
}

void draw_robot(int x0, int y0, double dir, int c1, int c2)
{
  int x1, y1;
  int x2, y2;
  int x3, y3;
  x1 = x0 + (int)(RR*cos(dir));
  y1 = y0 + (int)(RR*sin(dir));
  x2 = x0 + (int)(RR/4.0*cos(dir+M_PI_2));
  y2 = y0 + (int)(RR/4.0*sin(dir+M_PI_2));
  x3 = x0 + (int)(RR/4.0*cos(dir-M_PI_2));
  y3 = y0 + (int)(RR/4.0*sin(dir-M_PI_2));
  draw_line(x1, y1, x2, y2, c1, c2);
  draw_line(x2, y2, x3, y3, c1, c2);
  draw_line(x3, y3, x1, y1, c1, c2);
}


void delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit();
}

/* redraw of screen from pixmap */
gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_image(widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
		 image, 0, 0, 0, 0, XW, YW);
  return(TRUE);
}

gint draw(char thishostname[1024])
{
  int ret;
  int len;
  int xs, ys;
  fd_set rfds;
  struct timeval tv;
  char line[1024];
  char line0[1024];
  int i, j;
  int xa, ya, nry, nrb;
  int t;
  int min;
  double sec;
  char s[1024];
  int buflen;
  unsigned char *bf;
  int x, y;

  xa = XA; ya = YA;
  xs = WGOAL; ys = 0;
  update_rect.x = 0; update_rect.y = 0;
  update_rect.width = XW; update_rect.height = YW;

  sprintf(s, "%s %d Y %d\n", thishostname, RPORT, IDfirst);
  buflen = strlen(s);
  if (sendto(sfd, s, buflen, 0,
	     (struct sockaddr *)&saddr, sizeof(saddr)) != buflen){
    //    perror("sendto"); exit(1);
  }

  tv.tv_sec = 1; tv.tv_usec = 0;
  FD_ZERO(&rfds);
  FD_SET(rfd, &rfds);
  if (select(64, &rfds, (fd_set *)0, (fd_set *)0, &tv) < 0) {
    fprintf(stderr, "Warning: failed to select(2) in receiveUDPPacket().\n");
    return 0;
  }
  if(FD_ISSET(rfd, &rfds)){
    len = sizeof(caddr);
    if ((ret = recvfrom(rfd, line, 1024, 0, 
			(struct sockaddr *)&caddr, (socklen_t *)&len)) < 0){
      fprintf(stderr,
	      "Warning: failed to recvfrom(2) in receiveUDPPacket().\n");
      return 0;
    }
    line[ret] = '\0';
#ifdef DEBUG
    printf("%s",line);
#endif

    strcpy(line0, line);
    t = atoi(strtok(line, " \t\n"));
    xa = atoi(strtok(NULL, " \t\n"));
    ya = atoi(strtok(NULL, " \t\n"));
    bx = atoi(strtok(NULL, " \t\n"));
    by = atoi(strtok(NULL, " \t\n"));
    nry = atoi(strtok(NULL, " \t\n"));
    nrb = atoi(strtok(NULL, " \t\n"));
    xs = ((XA-xa)/2+WGOAL); ys = ((YA-ya)/2);
    for (i = 0; i < nry; i++){
      Yx[i] = atoi(strtok(NULL, " \t\n"));
      Yy[i] = atoi(strtok(NULL, " \t\n"));
      Ydir[i] = atoi(strtok(NULL, " \t\n")) * M_PI / 180.0;
    }
    for (i = 0; i < nrb; i++){
      Bx[i] = atoi(strtok(NULL, " \t\n"));
      By[i] = atoi(strtok(NULL, " \t\n"));
      Bdir[i] = atoi(strtok(NULL, " \t\n")) * M_PI / 180.0;
    }

    min = (int)((t*TFRAME/1000)/60);
    sec = t*TFRAME/1000.0 - (double)min*60.0;
    sprintf(s, "Frame#%5d, t=%d\"%05.2f'", t, min, sec);
    gtk_label_set(GTK_LABEL(label_info), s);
    /* -- drawing field, robots with markers and ball -- */

    /* erasing robots and ball in previous frame */

    draw_filled_box(0, 0, XA-1, YA-1, C_BACK1, C_BACK2);

    draw_filled_box(xs, ys, xa-1, ya-1, C_FIELD1, C_FIELD2);

    draw_cross(xs+bx, ya-by+ys, RB, RB, C_BALL1, C_BALL2);
    for (i = 0; i < nry; i++){
      draw_robot(xs+Yx[i], ya-Yy[i]+ys, Ydir[i], C_YPING1, C_YPING2);
#ifdef SHOW_ROBOT_NUMEBR_ON_PING_PONG
      for (j = 0; j < i+1; j++)
	draw_vline(Yx[i]+xs+2*j+2, ya-Yy[i]+ys-RM/2-4, 2, C_YPING1, C_YPING2);
#endif
    }

    for (i = 0; i < nrb; i++){
      draw_robot(xs+Bx[i], ya-By[i]+ys, Bdir[i], C_BPING1, C_BPING2);
#ifdef SHOW_ROBOT_NUMEBR_ON_PING_PONG
      for (j = 0; j < i+1; j++)
	draw_vline(Bx[i]+xs+2*j+2, ya-By[i]+ys-RM/2-4, 2, C_BPING1, C_BPING2);
#endif
    }

    draw_vline(xs, ys, ya, C_LINE1, C_LINE2);
    draw_vline((xs+xa), ys, ya, C_LINE1, C_LINE2);
    draw_hline(xs, xa, ys, C_LINE1, C_LINE2);
    draw_hline(xs, xa, (ys+ya), C_LINE1, C_LINE2);

    draw_vline((xa/2+xs), ys, ya, C_LINE1, C_LINE2);
    draw_filled_box(xs-WGOAL, (ys+ya/2-HGOAL/2), WGOAL-1, HGOAL,
		    C_YPING1, C_YPING2);
    draw_filled_box((xs+xa), (ys+ya/2-HGOAL/2),WGOAL-1, HGOAL,
		    C_BPING1, C_BPING2);

  }


  gtk_widget_draw(area, &update_rect);

  return TRUE;
}

void main(int ac, char **av)
{
  GtkWidget *window;
  GdkVisual *visual;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *label;
  GdkGCValues gc_values;
  GdkColormap *colormap;
  GdkColor color;
  int tag;
  int i;
  GtkWidget *toggle;
  FILE *fp;
  char buf[1024];
  char *thishostname;


  if (ac <4){
      fprintf(stderr, "usage: %s CCAM_HOST CCAM_PORT Y_ID_FIRST\n", av[0]);
      exit(1);
  }
  IDfirst = atoi(av[3]);

  /* socket for receiving */
  if ((rfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "Error: Failed to make socket for receiving.\n");
    exit(1);
  }
  bzero((char *)&sraddr, sizeof(sraddr));
  sraddr.sin_family = AF_INET;
  sraddr.sin_addr.s_addr = INADDR_ANY;
  sraddr.sin_port = htons(RPORT);
  if (bind(rfd, (struct sockaddr *)&sraddr, sizeof(sraddr)) < 0){
    fprintf(stderr, "Error: Failed to bind socket for receiving.\n");
    exit(1);
  }

  if ((sfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      perror("socket"); exit(1);
  }
  bzero((char *)&saddr, sizeof(saddr));
  if ((hp = gethostbyname(av[1])) == NULL) {
      perror("No such host"); exit(1);
  }
  bcopy(hp->h_addr, &saddr.sin_addr, hp->h_length);
  saddr.sin_family = AF_INET;
  saddr.sin_port = htons(atoi(av[2]));

  bx = XA/2*MAG; by = YA/2*MAG;

  thishostname = (char *)malloc(sizeof(char) * 1024);

  system("hostname > _tmp_");
  fp = fopen("_tmp_", "r"); fgets(buf, 1023, fp); fclose(fp);
  strcpy(thishostname, strtok(buf, " \t\n"));
  system("rm _tmp_");

  gtk_init(&ac, &av);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		     GTK_SIGNAL_FUNC(delete_event), NULL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy",
		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_window_set_title(GTK_WINDOW(window), "Status Viewer for cam");

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);

  label_info = gtk_label_new("Frame#    0, t=0\"0.00'");
  gtk_box_pack_start(GTK_BOX(vbox), label_info, TRUE, TRUE, 0);

  visual = gdk_visual_get_best_with_depth(16);
  if (visual == NULL) printf("Error: can't get visual for depth 16\n");
  image = gdk_image_new(GDK_IMAGE_NORMAL, visual, XW, YW);
  area = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(area), XW, YW);
  gtk_box_pack_start(GTK_BOX(vbox), area, TRUE, TRUE, 0);
  (((GdkImagePrivate*)image)->ximage)->data = (char *)xbuf;

  gtk_widget_draw(area, &update_rect);

  gtk_signal_connect(GTK_OBJECT(area), "expose_event",
		     (GtkSignalFunc)expose_event, NULL);

  gtk_widget_set_events(area, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  button = gtk_button_new_with_label("Quit");
  gtk_box_pack_start(GTK_BOX(vbox), button ,FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(delete_event), NULL);
  tag = gtk_timeout_add(TCYCLE, (GtkFunction)draw, (gpointer)thishostname);

  gtk_widget_show_all(window);

  gtk_main();

  gtk_timeout_remove(tag);
  close(rfd);
  close(sfd);
}
